#include "RS232_control.h"


RS232_control::RS232_control()
{
	port = "Com1";
	baudrate = 115200;				// sets baudrate
	parity_checking = 0;			// enable parity checking (1 = ON)
	RTS_hardware = 0;				// RTS flow control (0 = Hardware-Handshake)
	ByteSize = 8;					// number of bits/byte, 4-8 
	Parity = 0;						// 0-4=no,odd,even,mark,space 
	StopBits = 0;					// 0,1,2 = 1, 1.5, 2 

	timeoutEnabled = false;

	valid_ = open();
	init();
}


RS232_control::~RS232_control()
{
	close();
}


bool RS232_control::open(){

	hPort = CreateFile(port,		/* pointer to name of the port*/
		GENERIC_READ | GENERIC_WRITE,	/* access read-write mode*/
		0,							/* shared mode (0= off, 1=on)*/
		NULL,							/* handle can not be inherited to child-process*/
		OPEN_EXISTING,				/* open existing file- if file not exists --> error */
		0,							/* no attributes and flags */
		NULL);						/* Handle to file with attributes to copy */

	if(hPort != INVALID_HANDLE_VALUE)
	{							
		printf ("Connection established on Port: %s.\n", port);
		return true;
	}

	printf("Cannot open IO port %s (error %d)\n", port, GetLastError());
	return false;

}


bool RS232_control::read_data(unsigned char * inputData, int size, DWORD * length)
{	
	//printf("Start receiving...");
	if (ReadFile(hPort,  // handle of file to read
    inputData,         // handle of file to read
		size,              // number of bytes to read
		length,            // pointer to number of bytes read
		NULL) == 0)        // pointer to structure for data
	{
    printf("Read problem...\n");
		return FALSE;
	}	
	else
	{
		//printf("length of data received %i\n", *length);
		return true;
	}	
}

void RS232_control::init(){	
	hPort_ready = SetupComm(hPort,	// Size of the buffer
		512,		   				// size in bytes of input buffer
		512);						// size in bytes of output buffer

	//com1_state.fOutX = 0;
	//com1_state.fInX = 0;
	//com1_state.fDtrControl = DTR_CONTROL_DISABLE;
	//com1_state.fRtsControl = RTS_CONTROL_DISABLE;
	com1_state.BaudRate = CBR_115200;						// current baud rate 
	com1_state.ByteSize = DATABITS_8;						// number of bits/byte, 4-8 
	com1_state.StopBits = STOPBITS_10;
	com1_state.Parity = NOPARITY;							// 0-4=no,odd,even,mark,space 
	hPort_ready = SetCommState(hPort, &com1_state);	// Werte fuer rs232 setzen
	
	enableTimeout(false);
}	


bool RS232_control::enableTimeout(bool val){	
	if(val)
	{
		com1_timeouts.ReadIntervalTimeout = 10;					// max. time between two char
		com1_timeouts.ReadTotalTimeoutMultiplier = 1;			// multiplier * number of bytes = total_timeout (read)
		com1_timeouts.ReadTotalTimeoutConstant = 20;			// Total_timeout = Mult*n_bytes + Constant		(read)
		com1_timeouts.WriteTotalTimeoutMultiplier = 1;			// multiplier * number of bytes = total_timeout (write)
		com1_timeouts.WriteTotalTimeoutConstant = 1;			// Total_timeout = Mult*n_bytes + Constant		(write)	
		hPort_ready = SetCommTimeouts (hPort, &com1_timeouts);	// Werte fuer Kommunikationstimeout (ms)
		timeoutEnabled = true;
	}
	else
	{
		com1_timeouts.ReadIntervalTimeout = 0;					// max. time between two char
		com1_timeouts.ReadTotalTimeoutMultiplier = 0;			// multiplier * number of bytes = total_timeout (read)
		com1_timeouts.ReadTotalTimeoutConstant = 0;			// Total_timeout = Mult*n_bytes + Constant		(read)
		com1_timeouts.WriteTotalTimeoutMultiplier = 1;			// multiplier * number of bytes = total_timeout (write)
		com1_timeouts.WriteTotalTimeoutConstant = 1;			// Total_timeout = Mult*n_bytes + Constant		(write)	
		hPort_ready = SetCommTimeouts (hPort, &com1_timeouts);	// Werte fuer Kommunikationstimeout (ms)
		timeoutEnabled = false;
	}

	return hPort_ready;
}

bool RS232_control::write_data(unsigned char *data, int n_bytes){	
	transmit_status = WriteFile(hPort, /* write data to file specified by handler */
		data,													/* pointer to data to write to file */
		n_bytes,											/* number of bytes to write */
		&iBytesWritten,								/* pointer to number of bytes written */
		NULL);												/* pointer to structure for overlapped I/O */
	
	printf("%d Byte(s) written\n", iBytesWritten);

	if (!transmit_status) {
		printf("Cannot write message (error %d)\n", GetLastError());
		return false;
	}
	return true;
}

bool RS232_control::close(){
	transmit_status = CloseHandle(hPort);
	if (!transmit_status) {
		printf("Cannot close port handle (error %d)\n", GetLastError());
		return false;
	}
	
	printf("Connection closed\n");
	return true;
}	

