/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch/
 *
 */

 /**
 * @file bt_semaphore.c
 *
 * @brief Implements mutex and bt_semaphores
 * 
 *
 * 2004/05/07 14:07:09 Mathias Payer <payerm@student.ethz.ch>
 *
 */

#include <sys/event.h>
#include <sys/heap.h>
#include <bt/bt_semaphore.h>
#include <terminal/btn-terminal.h>

struct _bt_semaphore_list * head = NULL;

/*******************************************************************************
 * _bt_semaphore_init
 ******************************************************************************/
void _bt_semaphore_init(struct bt_semaphore *sema, int initial)
{
    sema->queue = 0;
    sema->counter = initial;
   
    if (head == NULL)
    {
        head = NutHeapAllocClear(sizeof(struct _bt_semaphore_list));
        head->sema = sema;
        //head->next = NULL;
    }
    else
    {
        struct _bt_semaphore_list * entry = head;
        while (entry->next != NULL)
        {
            entry = entry->next;
        }
        entry->next = NutHeapAllocClear(sizeof(struct _bt_semaphore_list));
        entry->next->sema = sema;
        //entry->next = NULL;
    }
}


/*******************************************************************************
 * _bt_semaphore_trywait
 ******************************************************************************/
int _bt_semaphore_trywait(struct bt_semaphore *sema)
{
    if (sema->counter != 0) {
        sema->counter--;
        return 0;
    }
    return -1;

}


/*******************************************************************************
 * _bt_semaphore_wait
 ******************************************************************************/
int _bt_semaphore_wait(struct bt_semaphore *sema)
{
    while (sema->counter == 0)
        NutEventWait(&(sema->queue), NUT_WAIT_INFINITE);
    sema->counter--;
    return 0;
}


/*******************************************************************************
 * _bt_semaphore_post
 ******************************************************************************/
int _bt_semaphore_post(struct bt_semaphore *sema)
{
    sema->counter++;
    if (sema->counter == 1) {
        NutEventPostAsync(&(sema->queue));
    }
    return 0;
}


/*******************************************************************************
 * _bt_semaphore_postval
 ******************************************************************************/
int _bt_semaphore_postval(struct bt_semaphore *sema, int val)
{
    int old = sema->counter;
    sema->counter = val;
    if (old == 0) {
        NutEventBroadcastAsync(&(sema->queue));
    }
    return 0;
}


/*******************************************************************************
 * _bt_semaphore_postinc
 ******************************************************************************/
int _bt_semaphore_postinc(struct bt_semaphore *sema, int val)
{
    int old = sema->counter;
    sema->counter += val;
    if (old == 0) {
        NutEventBroadcastAsync(&(sema->queue));
    }
    return 0;
}

void _bt_semaphore_cmd(char * arg)
{
    struct _bt_semaphore_list * entry = head;
    while (entry != NULL)
    {
        tprintf("sema %p: %d\n", entry->sema, entry->sema->counter);
        entry = entry->next;
    } 
}

void bt_semaphore_register_cmds(void)
{
    btn_terminal_register_cmd("sema", _bt_semaphore_cmd);
}
