// serial_hci.cpp : Defines the entry point for the console application.
//
#include <windows.h>
#include "conio.h"
#include <vector>
#include <string>
#include "Command.h"
#include "RS232_control.h"
#include <assert.h>
#include "Parser_hcidump/type.h"
#include "Parser_hcidump/parser.h"

#define VERBOSE

using namespace std;

typedef unsigned char byte;

typedef vector<byte> bVector;

#define DUMP_HDR_SIZE 20
#define SNAP_LEN 1024
// Liste des tailles des commandes reues/envoyes
#define GET_BD_ADDR_LEN 4
#define WSCAN_ENABLE_LEN 5
#define WSCAN_ACK_LEN 7
#define ACCEPT_PS3CONN_LEN 11
#define ACCEPT_PCCONN_LEN 11
#define CONNECTION_REQ_EVT_LEN 13
#define CONN_COMPLETE_EVT_LEN 14
#define L2CAP_CMD_LEN 10
#define L2CAP_CONN_REQU_LEN 17
#define L2CAP_RANDOM_LEN 13
#define ACCEPT_L2CAP_CONN_REQU_11_LEN 21
#define L2CAP_CONFIG_REQU_EVT_LEN 17
#define L2CAP_CONFIG_RESP_LEN 19
#define L2CAP_CONFIG_REQU_LEN 21
#define L2CAP_CONFIG_REQU_QOS_LEN 41
#define CMD_STATUS_EVT_LEN_1 7
#define CMD_STATUS_EVT_LEN_2 8
#define HCI_CHANGE_CONN_PACK_LEN 8
#define HCI_PAGE_SCAN_CHANGE_EVT_LEN 10
#define HID_SET_REPORT_LEN 15
#define HID_SET_BOOT_MODE_LEN 10 
#define HID_SET_OUTPUT_REPORT_LEN 12
#define HAND_SHAKE_LEN_15 15
#define HAND_SHAKE_LEN_10 10
#define HAND_SHAKE_LEN_22 22
#define HAND_SHAKE_LEN_64 59
#define HAND_SHAKE_LEN_8 8
#define DATA_LEN 50

// Liste des commandes Bluetooth

// Liste de divers "handshake", i.e. des commandes d'une longueur donnes mais dont le contenu n'est pas utile
Command HAND_SHAKE_8("HAND_SHAKE_LEN_8", HAND_SHAKE_LEN_8, 0x01);
Command HAND_SHAKE_15("HAND_SHAKE_15", HAND_SHAKE_LEN_15, 0x00, 0x00, 0x00, 0x00, 0x00);
Command HAND_SHAKE_10("HAND_SHAKE_10", HAND_SHAKE_LEN_10, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01);

// Buffer de Data reues. La longueur "devrait" tre  59, mais pour le moment elle est  22
Command DATA("DATA",DATA_LEN, 0x05);
Command HAND_SHAKE_22("HAND_SHAKE_LEN_22", HAND_SHAKE_LEN_22, 0x01);
Command HAND_SHAKE_64("HAND_SHAKE_LEN_64", HAND_SHAKE_LEN_64, 0x01);


Command GETBD_ADDR("GETBD_ADDR", GET_BD_ADDR_LEN,0x01,0x09,0x10,0x00);

// Activation de l'inquiry et du paging (connectable et dcouvrable)
Command WSCAN_ENABLE("WSCAN_ENABLE", WSCAN_ENABLE_LEN, 0x01,0x1A,0x0C,0x01,0x03);
Command WSCAN_ENABLE_ACK("WSCAN_ACK", WSCAN_ACK_LEN, 0x04, 0x0F, 0x04, 0x02, 0x01, 0x09, 0x04);

// Demande de connexion
// HACK : L'acceptation et la connexion doivent avoir un champ mac adress variable
Command ACCEPTPS3_CONN("ACCEPTPS3_CONN", ACCEPT_PS3CONN_LEN, 0x01,0x09,0x04,0x07,0xF8,0x62,0xE2,0xC1,0x19,0x00,0x01);
Command ACCEPTPS3_CONN_1("ACCEPTPS3_CONN", ACCEPT_PS3CONN_LEN, 0x01,0x09,0x04,0x07,0x55,0x4a,0x36,0xc1,0x19,0x00,0x01);//   0xF8,0x62,0xE2,0xC1,0x19,0x00,0x01);

Command CONNECTION_REQ_EVT("CONN_REQ_EVT", CONNECTION_REQ_EVT_LEN, 0x4, 0x4, 0x0A, 0x36, 0x36, 0xED, 0xA9, 0x13, 0x00, 0x00, 0, 0, 1);
Command CONN_COMPLETE_EVT("CONN_COMPLETE_EVT", CONN_COMPLETE_EVT_LEN, 0x4, 0x3, 0x0B,0x00,0x00,0x00,0x36,0x36,0xED,0xA9,0x13,0x00,0x01,0x00);

// Pour les tests sur pc
Command ACCEPTPC_CONN("ACCEPTPC_CONN", ACCEPT_PCCONN_LEN, 0x01,0x09,0x04,0x07,0x36,0x36,0xED,0xA9,0x13,0x00,0x01); 

// Les status des diverses commandes envoyes
Command CMD_STATUS_EVT_1("CMD_STATUS_EVT_1", CMD_STATUS_EVT_LEN_1, 0x4, 0x0F, 0x04, 0x00, 0x01, 0x09, 0x04);
Command CMD_STATUS_EVT_2("CMD_STATUS_EVT_2", CMD_STATUS_EVT_LEN_2, 0x4, 0x13, 0x05, 0x01, 0x2b, 0x00, 0x01,0x00);

// HACK : Commandes ncessaires pour activer les acclromtres ? J'en doute, mais  essayer
Command HCI_CHANGE_CONN_PACK("HCI_CHANGE_CONN_PACK", HCI_CHANGE_CONN_PACK_LEN, 0x01, 0x0f, 0x04, 0x04, 0x2b, 0x00, 0x18, 0xcc);
Command HCI_PAGE_SCAN_CHANGE_EVT("HCI_PAGE_SCAN_CHANGE_EVT",HCI_PAGE_SCAN_CHANGE_EVT_LEN,0x04, 0x20, 0x07, 0xf8, 0x62, 0xe2, 0xc1, 0x19, 0x00, 0x01);


//Command L2CAP_CMD("L2CAP_CMD", L2CAP_CMD_LEN, 0x02,0x00,0x20,0x09,0x00,0x05,0x00,0x12,0x00,0xDE);

// Connection request sur le PSM 11. Ceci est utilis dans la fonction receive_l2cap, mais c'est vraiment dgueu
// HACK : Revoir la fonction receive_l2cap
Command L2CAP_CONN_REQU_11_EVT("L2CAP_CONN_REQU_11_EVT",L2CAP_CONN_REQU_LEN, 0x02, 0x0, 0x20, 0x0c, 0x00, 0x08, 0x00, 0x01, 0x00, 0x02, 0x01, 0x4, 0x00, 0x11, 0x00, 0x45, 0x00);  

// Les deux demandes de configuration pour les deux PSM. Une ne demande rien et l'autre demande du QOS (quality of service)
Command L2CAP_CONFIG_REQU_EVT("L2CAP_CONFIG_REQU_EVT",L2CAP_CONFIG_REQU_EVT_LEN, 0x02, 0x0, 0x20, 0x0c, 0x00, 0x08, 0x00, 0x01, 0x00, 0x04, 0x02, 0x04, 0x00, 0x40,0x00, 0x00, 0x00);
Command L2CAP_CONFIG_REQU_QOS_EVT("L2CAP_CONFIG_REQU_EVT",L2CAP_CONFIG_REQU_QOS_LEN, 0x02, 0x0, 0x20, 0x0c, 0x00, 0x08, 0x00, 0x01, 0x00, 0x04, 0x02, 0x04, 0x00, 0x40,0x00, 0x00, 0x00);

//Command L2CAP_CONFIG_RESP("L2CAP_CONFIG_RESP", L2CAP_CONFIG_RESP_LEN, 0x02, 0x0, 0x20, 0x0c, 0x0, 0x0a, 0x0, 0x05, 0x02, 0x06, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00);
//Command L2CAP_CONFIG_REQU("L2CAP_CONFIG_REQU",L2CAP_CONFIG_REQU_LEN, 0x02, 0x0, 0x20, 0x10, 0x00, 0x0c,0x00, 0x01, 0x00, 0x04, 0x01, 0x08, 0x00, 0x40, 0x00, 0x00, 0x00, 0x01, 0x02, 0x40, 0x00);
//																																		02     2c   20     10   00    0c   00    01   00     04    01    08   VAR    VAR   00    00    00    01    02    40    00
//Command ACCEPT_L2CAP_CONN_REQU_11("ACCEPT_L2CAP_CONN_REQU_11",ACCEPT_L2CAP_CONN_REQU_11_LEN, 0x02, 0x0, 0x20, 0x10, 0x00, 0x0c, 0x00, 0x01, 0x00, 0x03, 0x01, 0x08, 0x00, 0x40, 0x00, 0x45, 0x00, 0x00, 0x00, 0x00, 0x00 );
//Command L2CAP_RANDOM("L2CAP_RANDOM", L2CAP_RANDOM_LEN, 0x02, 0x00, 0x20, 0x08, 0x00, 0x04, 0x00, 0x10, 0x00, 0x61, 0x62, 0x61, 0x62);

// Commandes d'activation de la PS3

//Command L2CAP_PS3_CONN("L2CAP_PS3_CONN",17,0x01,0x5,0x04,0x0D, 0x36, 0x36, 0xED, 0xA9, 0x13, 0x00, 0x08, 0x00, 00, 00, 00, 00,01);
																					// 01  05    04   0D   36    36    ED    A9    13    00    08    00   00 00  00  00 00

RS232_control * rs232;		

void process_frames(char *data, int length)
{
//	struct cmsghdr *cmsg;
//	struct msghdr msg;
//	struct iovec  iv;
//	struct dump_hdr *dh;

	struct frame frm;
	char *buf, *ctrl;
	int snap_len = 0;

	if (snap_len < SNAP_LEN)
		snap_len = SNAP_LEN;

	if (!(buf = (char*)malloc(snap_len + DUMP_HDR_SIZE))) {
		perror("Can't allocate data buffer");
		exit(1);
	}
	//dh = (dump_hdr *) buf;
	frm.data = buf + DUMP_HDR_SIZE;
	
	if (!(ctrl = (char*)malloc(100))) {
		perror("Can't allocate control buffer");
		exit(1);
	}
	
//	printf("device: %s snap_len: %d filter: 0x%lx\n", 
//		dev? dev : "any", snap_len, filter);
//
//	if (getuid() != 0)
//		printf(
//"** WARNING: You are running hcidump as non-privileged user.\n" \
//"** WARNING: You will not be able to see all data.\n\n");

	//memset(&msg, 0, sizeof(msg));

	/*while (1) {
		iv.iov_base = frm.data;
		iv.iov_len  = snap_len;

		msg.msg_iov = &iv;
		msg.msg_iovlen = 1;
		msg.msg_control = ctrl;
		msg.msg_controllen = 100;*/

		/*if ((frm.data_len = recvmsg(sock, &msg, 0)) < 0) {
			perror("Receive failed");
			exit(1);
		}*/
	  frm.data_len = length;


		/* Process control message */
		frm.in = 0;
		//cmsg = CMSG_FIRSTHDR(&msg);
		/*while (cmsg) {
			if (cmsg->cmsg_level == SOL_HCI_RAW &&
			    cmsg->cmsg_type == SCM_HCI_RAW_DIRECTION)
				memcpy(&frm.in,CMSG_DATA(cmsg),sizeof(frm.in));

			if (cmsg->cmsg_level == SOL_SOCKET &&
			    cmsg->cmsg_type == SCM_TIMESTAMP)
				memcpy(&frm.ts,CMSG_DATA(cmsg),sizeof(frm.ts));

			cmsg = CMSG_NXTHDR(&msg, cmsg);
		}*/
		memcpy(&frm.in,data,sizeof(frm.in));

		frm.ptr = frm.data;
		frm.len = frm.data_len;

		//switch (mode) {
		//case WRITE:
		//	/* Save dump */	
		//	dh->len = htole16(frm.data_len);
		//	dh->in  = frm.in;
		//	dh->ts_sec  = htole32(frm.ts.tv_sec);
		//	dh->ts_usec = htole32(frm.ts.tv_usec);
		//	if (write_n(file, buf, frm.data_len + DUMP_HDR_SIZE) < 0) {
		//		perror("Write error");
		//		exit(1);
		//	}
		//	break;

		//default:
			/* Parse and print */
			parse(&frm);
			//break;
			
}


// Commande d'envoi standard
bool send_command(Command cmd) {
	byte buffer[255];	


	cout << ">> Sending " << cmd.toString() << " - ";
		
	for(int i = 0; i<cmd.getLength(); i++){		
		if(i != cmd.getLength()-1)
			printf("0x%X:", cmd.getCommand()[i]);
		else
			printf("0x%X", cmd.getCommand()[i]);
	}


	printf("\n\t");


	for(int i = 0; i < cmd.getLength(); i++)
	{
		buffer[i] = cmd.getCommand()[i];
	}

	return rs232->write_data(buffer, cmd.getLength());
	
}


// Commande de parsing pour trouver la longueur du paquet
bool size_finder(byte buffer[], int* size_complete, int* size_header, DWORD* length) {
	// Machine d'tat
  int	size_test = 0;
	int etat = 0;
	while (etat!=999) {
		switch(etat)  {
			
			// Etat de dpart : lecture du premier bit et vrification du type de paquet (04 ou autres)
			case 0 : rs232->read_data(buffer, 1, length);
							 if (buffer[0]==4)  etat = 1;
							 else etat = 2;
							 size_test++;
							 printf("0x%X:", buffer[0]);
							 //printf("Premier byte : 0x%x \n",buffer[0]);
							 //printf("Taille : %d \n", size_test);
				break;
			// Notre header vaut 04, il est donc suivi d'un autre byte puis de la taille
			case 1 : rs232->read_data(&buffer[1], 2, length);
				// La taille est dans le header[2]
				// +2 pour les deux paquets lu (le second byte et la taille)
				*size_complete = size_test+2+buffer[2];
				*size_header = 3;
        printf("0x%X:", buffer[1]);
				printf("0x%X:", buffer[2]);

				// Sortie de la machine d'tat
				//printf("Taille : %d \n", size_test);
				etat = 999;
				break;
				// Notre header vaut 02, il est donc suivi de deux paquets puis de (taille-1) du reste
			case 2 : rs232->read_data(&buffer[1],3,length);
				*size_complete = size_test+4+buffer[3];
				printf("0x%X:", buffer[1]);
				printf("0x%X:", buffer[2]);
				printf("0x%X:", buffer[3]);
				*size_header = 4;
				// sortie de la machine
				etat = 999;
				break;
		
			}
	}
	return true;
}

// Commande de rception standard
bool receive_command(Command cmd) {
#ifdef VERBOSE
	cout << "<< Receiving " << cmd.toString() << " - NBytes " << cmd.getLength() << "\n\t";
#endif VERBOSE
	byte buffer[1024];
	int size = cmd.getLength(); 
	int size_header = 0;
	int size_test = 0;
	int size_complete = 0;
	DWORD length;


	// METTRE COMMANDE ICI
	size_finder(buffer, &size_complete, &size_header, &length);
	
	if(!rs232->read_data(&buffer[size_header], size_complete-size_header, &length))
	{
		printf("Error during read\n");
		return 0;
	}	
	assert(length == size_complete-size_header);
#ifdef VERBOSE
	//printf("Event : ");
#endif VERBOSE
	for(DWORD i = 0; i<length; i++){		
		if(i != length -1)
			printf("0x%X:", buffer[i]);
		else
			printf("0x%X", buffer[i]);
	}
	printf("\nLongueur totale : %d \n",size_complete);
	printf("Longueur restante a lire : %d \n", size_complete-size_header);
	process_frames((char*)buffer, size_complete-size_header);

	printf("\n");	
	return 1;

}

// Commande de rception pour les data brutes (data hid)
bool receive_data(Command cmd) {
  byte buffer[2048];
	int size = cmd.getLength(); 	
	DWORD length;

	if(!rs232->read_data(buffer, size, &length))
	{
		printf("Error during read\n");
		return 0;
	}	
	
	if(!rs232->isTimeoutEnabled())
	{
		assert(length == cmd.getLength());
	}
		
	for(DWORD i = 0; i<length; i++){		
		if(i != length -1)
			printf("%.2X:", buffer[i]);
		else
			printf("%.2X", buffer[i]);
	}

	printf("\r");
	//printf("\n\n");	
	return 1;

}

// Commande de rception L2cap pour enregistrer les paramtres de CID (Cannal id)
Command receive_l2cap(Command cmd, byte* cid1, byte* cid2) {
	
	cout << "<< Receiving " << cmd.toString() << " - NBytes " << cmd.getLength() << "\n\t";

	byte buffer[1024];
	int size = cmd.getLength(); 	
	DWORD length;

	if(!rs232->read_data(buffer, size, &length))
	{
		printf("Error during read\n");
		exit(-1);
	}	
	assert(length == cmd.getLength());

	printf("Event : ");
	for(DWORD i = 0; i<length; i++){		
		if(i != length -1)
			printf("0x%X:", buffer[i]);
		else
			printf("0x%X", buffer[i]);
	}
	printf("Valeurs variables : 0x%X 0x%X\n", buffer[14], buffer[15]);	
	*cid1 = buffer[14];
	*cid2 = buffer[15];
	
	Command l2cap("ACCEPT_L2CAP_CONN_REQU_11",ACCEPT_L2CAP_CONN_REQU_11_LEN, 0x02, 0x0, 0x20, 0x10, 0x00, 0x0c, 0x00, 0x01, 0x00, 0x03, 0x01, 0x08, 0x00, 0x40, buffer[14], buffer[15], 0x00, 0x00, 0x00, 0x00, 0x00 );
	printf("\n");	
	return l2cap;

}

// Commande de rception pour les connections acl (mac variable)
bool receive_l2cap(Command cmd, byte* mac0, byte* mac1, byte* mac2, byte* mac3, byte* mac4, byte* mac5 ) {
	
	cout << "<< Receiving " << cmd.toString() << " - NBytes " << cmd.getLength() << "\n\t";

	byte buffer[1024];
	int size = cmd.getLength(); 	
	DWORD length;

	if(!rs232->read_data(buffer, size, &length))
	{
		printf("Error during read\n");
		exit(-1);
	}	
	assert(length == cmd.getLength());

	printf("Event : ");
	for(DWORD i = 0; i<length; i++){		
		if(i != length -1)
			printf("0x%X:", buffer[i]);
		else
			printf("0x%X", buffer[i]);
	}
	printf("Valeurs variables : 0x%X 0x%X\n", buffer[14], buffer[15]);	
	
	printf("\n");	
	return 1;

}

// Pretty printer pour les frames qu'on transmet : 
// tir et adapt de hcidump de Linux



    
int main(int argc, char * argv[])
{	
	//unsigned char buf[255];
	//unsigned char rcv_buff[255];
	int i = 0; 	
	DWORD length = 0;		

	// Initialisation du parser de hcidump
	init_parser(DUMP_ASCII, ~0L, 0,0,-1,-1);

	rs232 = new RS232_control();

	if(!rs232->valid()){
		exit(-1);
	}
		

	send_command(WSCAN_ENABLE); 	
	receive_command(WSCAN_ENABLE_ACK);

// Test sur la console PS3

	//printf("\nSending connection request\n\t");
	//send_command(L2CAP_PS3_CONN);
	//receive_data(HAND_SHAKE_22);
	// Ci-dessous, la connexion  la manette



	// Wait for incoming connection from the sixaxis
	printf("\nWaiting for incoming connection...\n\t");
	receive_command(CONNECTION_REQ_EVT);
	
	// Accept de la connexion pc	
	//printf("\nSending ACCEPTPC_CONN\n");
	//send_command(ACCEPTPC_CONN);	

	//Accept de la connexion manette
	printf("\nSending ACCEPTPS3_CONN\n");
	send_command(ACCEPTPS3_CONN);
	//send_command(ACCEPTPS3_CONN_1);
	
	// Command status event
	printf("Getting command status event\n\t");	
	receive_command(CMD_STATUS_EVT_1);	

	// Connection complete event
	printf("Getting connection complete event\n\t");	
	receive_command(CONN_COMPLETE_EVT);	

	// Send change packet type
	//printf("Send change packet type \n\t");
	//send_command(HCI_CHANGE_CONN_PACK);

	// Receive Page scan repetition mode change
	//printf("Receive Page scan repetition mode change \n\t");
	//receive_command(HCI_PAGE_SCAN_CHANGE_EVT);


	//!!!!!!!!!!!!!!!!! 
	//!!!!!!!!!!!!!!!!! Exprimental
	//printf("Sending policy to change link role \n"); 
	//Command WRITE_LINK_POLICY("WRITE_LINK_POLICY",8, 0x02, 0x0, 0x20, 0x10, 0x00, 0x0c,0x00, 0x01, 0x00, 0x04, 0x01, 0x08, cid1, cid2, 0x00, 0x00, 0x00, 0x01, 0x02, 0x40, 0x00);



	// Receive l2cap connection request
	printf("Getting l2cap connection request\n\t");
	byte cid1, cid2;

	Command ACCEPT_L2CAP_CONN_REQU_11 = receive_l2cap(L2CAP_CONN_REQU_11_EVT, &cid1, &cid2);
	printf("Cid : 0x%X%X \n", cid1, cid2);
	// Create config response packet
	// FAUX
	//Command L2CAP_CONFIG_RESP("L2CAP_CONFIG_RESP", L2CAP_CONFIG_RESP_LEN, 0x02, 0x0, 0x20, 0x0e, 0x0, 0x0a, 0x0, 0x05, 0x02, 0x06, cid1,cid2, 0x00, 0x00, 0x00, 0x00, 0x00);
	// JUSTE
	Command L2CAP_CONFIG_RESP("L2CAP_CONFIG_RESP", L2CAP_CONFIG_RESP_LEN, 0x02, 0x0, 0x20, 0x0e, 0x0, 0x0a, 0x0, 0x01, 0x00, 0x05, 0x02, 0x06, cid1,cid2, 0x00, 0x00, 0x00, 0x00, 0x00);
	//                                                                      0x02  0x2c 0x20   0x0e 0x0  0x0a  0x0  0x01  0x00  0x05 0x02 0x06 0x00 0x44 0x00 0x00 0x00 0x00 0x00
	// Create config request packet
Command L2CAP_CONFIG_REQU("L2CAP_CONFIG_REQU",L2CAP_CONFIG_REQU_LEN, 0x02, 0x0, 0x20, 0x10, 0x00, 0x0c,0x00, 0x01, 0x00, 0x04, 0x01, 0x08, cid1, cid2, 0x00, 0x00, 0x00, 0x01, 0x02, 0x40, 0x00);
//																																		02     2c   20     10   00    0c   00    01   00     04    01    08   VAR    VAR   00    00    00    01    02    40    00
	

	// Sending l2cap connection accept
	printf("Sending l2cap connection accept\n\t");
	send_command(ACCEPT_L2CAP_CONN_REQU_11);
	
	// Receive config request
	printf("Receive config request ! (fete du slip)\n\t");
	receive_command(L2CAP_CONFIG_REQU_EVT);

	// Sending config response
	printf("Sending config response \n\t");
	send_command(L2CAP_CONFIG_RESP);

	// Sending a config request
	printf("Sending config request \n\t");
	send_command(L2CAP_CONFIG_REQU);

	// Receive a config response
	printf("Receive config response \n\t");
	receive_command(L2CAP_CONFIG_RESP);

	// Receive Command status
	//printf("Receive command status \n\t");
	//receive_command(CMD_STATUS_EVT_2);

	// Receive 2nd l2cap connection (with variable scid!)
	printf("Receive 2nd l2cap connection request \n\t");
	
	// Prendre les nouvelles valeurs des dcid
	byte did1, did2;
	receive_l2cap(L2CAP_CONN_REQU_11_EVT, &did1, &did2);
	// Create ACCEPT_L2CAP_CONN_REQU_13
	//Fonctionnelle
	//Command ACCEPT_L2CAP_CONN_REQU_13("ACCEPT_L2CAP_CONN_REQU_13",ACCEPT_L2CAP_CONN_REQU_11_LEN, 0x02, 0x0, 0x20, 0x10, 0x00, 0x0c,0x00, 0x01, 0x00, 0x03, 0x03, 0x08, did1, did2, 0x00, 0x45, 0x00, 0x00, 0x00, 0x00);
  //Test																																												 02    2c    20   10    00    0c   00    01    00    03    03    08   VAR   VAR    00    45    00    00    00    00
	Command ACCEPT_L2CAP_CONN_REQU_13("ACCEPT_L2CAP_CONN_REQU_13",ACCEPT_L2CAP_CONN_REQU_11_LEN, 0x02, 0x0, 0x20, 0x10, 0x00, 0x0c,0x00, 0x01, 0x00, 0x03, 0x03, 0x08, 0x00, 0x41, did1, did2, 0x00, 0x00, 0x00, 0x00);

	printf("Cid : 0x%X%X \n", did1, did2);
	
	// Receive Command status
	printf("Receive command status \n\t");
	receive_command(CMD_STATUS_EVT_2);

		// Sending l2cap connection accept
	printf("Sending l2cap connection accept\n\t");
	send_command(ACCEPT_L2CAP_CONN_REQU_13);
	
	// Receive Command status
	//printf("Receive command status \n\t");
	//receive_command(CMD_STATUS_EVT_2);


	// Receive config request
	printf("Receive BIG config request ! (fete du slip)\n\t");
	receive_command(L2CAP_CONFIG_REQU_QOS_EVT);
	
	// Create config response to qos
	Command L2CAP_CONFIG_RESP_QOS("L2CAP_CONFIG_RESP_QOS", L2CAP_CONFIG_RESP_LEN, 0x02, 0x0, 0x20, 0x0e, 0x0, 0x0a, 0x0, 0x01, 0x00, 0x05, 0x04, 0x06, did1,did2, 0x00, 0x00, 0x00, 0x00, 0x00);
	//                                                                      0x02  0x2c 0x20   0x0e 0x0  0x0a  0x0  0x01  0x00  0x05 0x02 0x06 0x00 0x44 0x00 0x00 0x00 0x00 0x00
	// Create config request packet
	Command L2CAP_CONFIG_REQU_QOS("L2CAP_CONFIG_REQU_QOS",L2CAP_CONFIG_REQU_LEN, 0x02, 0x0, 0x20, 0x10, 0x00, 0x0c,0x00, 0x01, 0x00, 0x04, 0x02, 0x08, did1, did2, 0x00, 0x00, 0x00, 0x01, 0x02, 0x40, 0x00);
	//																																						 02    2c   20    10     00   0c   00    01    00    04    02    08   cid   cid    00    00    00    01    02    40    00

		// Sending config response
	printf("Sending config response \n\t");
	send_command(L2CAP_CONFIG_RESP_QOS);

	// Sending a config request
	printf("Sending config request \n\t");
	send_command(L2CAP_CONFIG_REQU_QOS);

	// Receive a config response
	printf("Receive config response \n\t");
	receive_command(L2CAP_CONFIG_RESP);
	
	// Receive Command status
	printf("Receive command status \n\t");
	receive_command(CMD_STATUS_EVT_2);


	// HID
	
	//Cration des trois commandes
	Command HID_SET_REPORT("HID_SET_REPORT", 14, 0x02, 0x00, 0x20, 0x0A, 0x00, 0x06, cid1, cid2,0x00,  0x53, 0xF4, 0x42, 0x03, 0x0, 0x0); // F4, 42, 03, 0, 0
  Command HID_SET_BOOT_MODE("HID_SET_BOOT_MODE", HID_SET_BOOT_MODE_LEN, 0x02, 0x00, 0x20, 0x05, 0x00, 0x01, cid1, cid2, 0x01, 0x70);
	Command HID_SET_OUTPUT_REPORT("HID_SET_OUTPUT_REPORT", HID_SET_OUTPUT_REPORT_LEN, 0x02, 0x00, 0x20, 0x07, 0x00, 0x03,did1 ,did2 , 0x00, 0xa2, 0x01, 0x00);

	//Sending
	send_command(HID_SET_REPORT);
	send_command(HID_SET_BOOT_MODE);
	send_command(HID_SET_OUTPUT_REPORT);

	


	char report = 0;

	rs232->enableTimeout(true);
	Sleep(1000);
	int j = 1;

	//while(1)
	//{
	//	if(_kbhit())
	//	{
	//		if(j == 1)
	//		{
	//			j= 0;
	//			char got = _getch();
	//			bool valid = false;
	//			
	//			if(got == '+')
	//			{
	//				report++;
	//				valid = true;
	//			}
	//			else if(got == '-')
	//			{
	//				report--;
	//				valid = true;
	//			}
	//			else if(got == '*')
	//			{
	//				valid = true;
	//			}				
	//			else if(isdigit(got))
	//			{
	//				printf("Got it!\n");
	//				int a = atoi(&got);
	//				report = a;	
	//				valid = true;
	//			}
	//			else
	//			{
	//				printf("Unrecognized command (+/-) valid only\n");					
	//			}

	//			if(valid)
	//			{
	//			printf("Sending new report\n");				
	//			Command HID_SET_REPORT("HID_SET_REPORT", HID_SET_REPORT_LEN, 0x02, 0x00, 0x20, 0x0A, 0x00, 0x06, cid1, cid2, 0x00, 0x53, 0xF4, 0x42, 0x03, 0x0); // F4, 42, 03, 0, 0
	//			send_command(HID_SET_REPORT);
	//			}
	//		}
	//		else
	//		{
	//			j = 1;
	//		}
	//	}
	//
	//	printf("\n\tReceiving data :");

	//	
	//}
	
	//0x03
	//Command HID_SET_BOOT_MODE("HID_SET_BOOT_MODE", HID_SET_BOOT_MODE_LEN, 0x02, 0x00, 0x20, 0x05, 0x00, 0x01, cid1, cid2, 0x01, 0x70);
	//Command HID_SET_OUTPUT_REPORT("HID_SET_OUTPUT_REPORT", HID_SET_OUTPUT_REPORT_LEN, 0x02, 0x00, 0x20, 0x07, 0x00, 0x03,did1 ,did2 , 0x00, 0xa2, 0x01, 0x00);

	//Sending
	
	//send_command(HID_SET_BOOT_MODE);
	// send_command(HID_SET_OUTPUT_REPORT);
	
		// Receive Command status
	//printf("Receive command status \n\t");
//	receive_command(CMD_STATUS_EVT_2);

	// Receive Command status
	printf("Receive command status \n\t");
	//receive_command(HAND_SHAKE_10);
	//receive_command(HAND_SHAKE_10);

	// Receive... ?
	printf("Suite \n\n\n\n\t");
	//!!!!!!!!!!!!!!!!! 
	//!!!!!!!!!!!!!!!!! Exprimental
	//printf("Sending led \n"); 
	/*Command WRITE_LED("WRITE_LED",50,0x02, 0x00, 0x20, 0x0A, 0x00, 0x06, did1, did2, 0x00,0x52,0x01,
																0x00,0x00,0x00,0xff,0x72,0x00,0x00,0x00,
																0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,
																0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
																0x00,0xff,0x27,0x10,0x32,0x32,0x00,0x00,
																0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00);*/
	
	
	//			Command HID_SET_REPORT("HID_SET_REPORT", HID_SET_REPORT_LEN, 0x02, 0x00, 0x20, 0x0A, 0x00,   0x06, cid1, cid2, 0x00,  0x53, 0xF4, 0x42, 0x03, 0x0); // F4, 42, 03, 0, 0
	//send_command(WRITE_LED);


	while(1) {
	receive_data(HAND_SHAKE_22);
	//receive_data(DATA);
	
	}

	while(1)
	{	
		if(_kbhit())
		{
			break;
		}		
	}
		
	return 0;
}
